<?php
/**
 * Created by PhpStorm.
 * User: RSkaldin
 * Date: 6/4/2019
 * Time: 9:20 PM
 */

namespace App\Http\Controllers\Postman;


use App\Http\Controllers\Controller;
use App\Models\Activity;
use App\Models\Address;
use App\Models\Client;
use App\Models\Orders;
use App\Models\Package;
use App\Models\PackageItems;
use App\Models\Product;
use Illuminate\Support\Facades\DB;
use PDF;

class DeclarationController extends Controller
{
    public function index()
    {
        $packages = Package::orderBy('id', 'DESC')->paginate(25);
        $declarationJSON = $declarationJson = $this->makeDeclarationJSON($packages);

        return view('pages.admin.declarations.index', [
            'packages' => $packages,
            'declarationJSON' => $declarationJSON,
        ]);
    }

    /**
     * Generate and download declaration in PDF format
     *
     * @param $package_id
     * @return \Illuminate\Http\Response
     */
    public function generatePDF($package_id)
    {
        $orderId = $package_id;


        $orderData = Orders::select([
            'id',
            'name',
            'client_id',
            'added',
            'track_number',
            'from_address',
            'to_address_id',
            'edited',
            DB::raw('(SELECT concat(firstname,\' \', secondname) FROM address_info where address_info.id=orders.to_address_id) as r_name'),
            DB::raw('(SELECT name FROM payment_statuses where payment_statuses.id=(select payment_statuses_id from payment where payment.order_id=orders.id)) as payment_status'),
            'amount',
            'weight',
            'summa',
        ])->where('id', '=', $orderId)->get();

        $clientData = Client::find($orderData[0]->client_id);
        $recipient = Address::select([
            'id',
            'user_id',
            'scan_id',
            'firstname',
            'secondname',
            'fathername',
            'phone',
            'street',
            'house',
            'passport',
            'passport_by',
            'passport_issue',
            'apartment',
            'zip',
            'is_default',
            'type',
            'created',
            DB::raw('(select title_uz from country where id = address_info.country) as country'),
            DB::raw('(SELECT (sum(orders.amount)-1000)*(-1) FROM orders where orders.to_address_id=address_info.id) as r_limit'),
            DB::raw('(select title_uz from city where id = address_info.city) as city')
        ])->where([
            ['id', '=', $orderData[0]->to_address_id]
        ])->get();
        $sender = Address::select([
            'id',
            'user_id',
            'scan_id',
            'firstname',
            'secondname',
            'fathername',
            'phone',
            'street',
            'house',
            'passport',
            'passport_by',
            'passport_issue',
            'apartment',
            'zip',
            'is_default',
            'type',
            'created',
            DB::raw('(select title_uz from country where id = address_info.country) as country'),
            DB::raw('(SELECT (sum(orders.amount)-1000)*(-1) FROM orders where orders.to_address_id=address_info.id) as r_limit'),
            DB::raw('(select title_uz from city where id = address_info.city) as city')
        ])->where([
            ['id', '=', $orderData[0]->from_address]
        ])->get();

        $products = Product::select([
            'id',
            'operator_id',
            'title',
            'amount',
            'weight',
            'price',
            'order_id',
        ])->where('order_id', '=', $orderData[0]->id)->get();

//        return view('templates.declaration', [
//            'title' => 'Declaration #',
//            'client' => $clientData,
//            'order' => $orderData[0],
//            'recipient' => $recipient[0],
//            'sender' => $sender[0],
//            'product' => $products
//        ]);
        $pdf = \Barryvdh\DomPDF\Facade::loadView('templates.declaration', [
            'title' => 'Declaration #',
            'client' => $clientData,
            'order' => $orderData[0],
            'recipient' => $recipient[0],
            'sender' => $sender[0],
            'product' => $products
        ])->setPaper('a4');

        $path = 'uploads/declarations/';
        $filename = md5(time()) . date('Ymdhis') . '.pdf';
        $pdf->save($path . $filename);
//        Activity::appendLog("Сгенерирована декларация в PDF формате.", Activity::NOTICE);
        return redirect(url($path . $filename));
//        return redirect(url($path . $filename));
//        return response()->json(['url' => url($path . $filename) . ''], 200);
//        return $pdf->download('declaration.pdf');
    }

    /**
     * Generate Declaration about packages
     *
     * @param $packages
     * @return string
     */
    private function makeDeclarationJSON(&$packages) : string
    {
        $declaration_data = [];
        foreach ($packages as $package) {
            $package_items = PackageItems::where('package_id', $package->id)->get();
            $items_data = [];
            foreach ($package_items as $item) {
                $items_data[] =[
                    'package_id' => $package->id,
                    'name' => $item->item_name,
                    'amount' => $item->amount,
                    'price' => $item->price,
                    'weight' => $item->weight,
                    'date' => $item->date,
                ];
            }
            $declaration_data[$package->id] = $items_data;
        }

        return json_encode($declaration_data);
    }
}
